import subprocess
from pathlib import Path

import pytest

from pytest_logikal.file_checker import FileCheckItem, FileCheckPlugin
from pytest_logikal.plugin import ItemRunError


def pytest_addoption(parser: pytest.Parser) -> None:
    group = parser.getgroup('docs')
    group.addoption('--docs', action='store_true', default=False, help='run docs checks')


def pytest_configure(config: pytest.Config) -> None:
    config.addinivalue_line('markers', 'docs: tests package docs.')
    if config.option.docs:
        config.pluginmanager.register(DocsPlugin(config=config))


class DocsItem(FileCheckItem):
    def runtest(self) -> None:
        process = subprocess.run(  # nosec: secure, not using untrusted input
            ['docs', '--build', '--clear'], capture_output=True, text=True, check=False,
        )
        if process.returncode != 0:
            raise ItemRunError(process.stderr or process.stdout)


class DocsPlugin(FileCheckPlugin):
    name = 'docs'
    item = DocsItem

    def check_file(self, file_path: Path) -> bool:
        return file_path.relative_to(self.config.invocation_params.dir) == Path('docs/index.rst')



from abc import abstractmethod
from pathlib import Path
from typing import Any, Dict, Iterable, Type, final

import pytest
from xdist import is_xdist_worker
from xdist.workermanage import WorkerController

from pytest_logikal.plugin import Item, Plugin


class FileCheckItem(Item):
    def __init__(self, *, plugin: 'FileCheckPlugin', **kwargs: Any):
        super().__init__(**kwargs)
        self.plugin = plugin

    @abstractmethod
    def runtest(self) -> None:
        ...


class FileCheckPlugin(Plugin):
    item: Type[FileCheckItem]

    def check_file(self, file_path: Path) -> bool:
        return file_path.suffix == '.py'

    # Note: the arguments change but that is fine because pytest dynamically prunes them
    def pytest_collect_file(  # type: ignore[override] # pylint: disable=arguments-differ
        self, file_path: Path, parent: pytest.Collector,
    ) -> Any:
        plugin = self
        if self.check_file(file_path):
            class File(pytest.File):
                def collect(self) -> Iterable[FileCheckItem]:
                    yield plugin.item.from_parent(parent=self, name=plugin.name, plugin=plugin)
            return File.from_parent(parent, path=file_path)
        return None


class CachedFileCheckItem(FileCheckItem):
    def __init__(self, *args: Any, **kwargs: Any):
        super().__init__(*args, **kwargs)
        self.plugin: 'CachedFileCheckPlugin'
        self._current_mtime: float

    def setup(self) -> None:
        previous_mtime = self.plugin.mtimes.get(str(self.path))
        self._current_mtime = self.path.lstat().st_mtime
        if previous_mtime is not None and previous_mtime == self._current_mtime:
            pytest.skip('file has previously passed check')

    @abstractmethod
    def run(self) -> None:
        ...

    @final
    def runtest(self) -> None:
        # Run the test in the child object
        self.run()

        # Store the file modification time if the test was successful
        self.plugin.new_mtimes[str(self.path)] = self._current_mtime


class CachedFileCheckPlugin(FileCheckPlugin):
    item: Type[CachedFileCheckItem]

    def __init__(self, config: pytest.Config):
        super().__init__(config=config)

        if not config.cache:
            raise RuntimeError('Cannot use a file check plugin without a cache')

        self.cache = config.cache
        self.mtimes_path = f'{self.name}/mtimes'
        self.mtimes = self.cache.get(self.mtimes_path, {})
        self.new_mtimes: Dict[str, float] = {}

    def pytest_testnodedown(self, node: WorkerController, *_args: Any, **_kwargs: Any) -> None:
        # Update the controller's file modification times with the values on the worker nodes
        self.new_mtimes.update(node.workeroutput[self.mtimes_path])

    def pytest_sessionfinish(self, session: pytest.Session, *_args: Any, **_kwargs: Any) -> None:
        if is_xdist_worker(session):
            # Transfer the new file modification times from the worker nodes to the controller node
            mtimes_path = self.mtimes_path
            self.config.workeroutput[mtimes_path] = self.new_mtimes  # type: ignore[attr-defined]
        else:
            # Update the cache with the new file modification times
            self.cache.set(self.mtimes_path, {**self.mtimes, **self.new_mtimes})



import inspect
import logging
import re
from contextlib import contextmanager
from dataclasses import asdict, dataclass
from operator import itemgetter
from pathlib import Path
from typing import Any, Callable, Dict, Iterable, Iterator, Optional, cast

import pytest
from selenium.common.exceptions import SessionNotCreatedException
from selenium.webdriver.chrome.options import Options as ChromeOptions
from selenium.webdriver.chrome.webdriver import WebDriver as Chrome

from pytest_logikal.utils import Function, assert_image_equal

logger = logging.getLogger(__name__)

Fixture = Callable[[Function], Function]


@pytest.fixture(scope='session', autouse=True)
def faker_seed() -> int:
    """
    Set a default seed for Faker for deterministic testing. Automatically applied.
    """
    return 42


@dataclass(frozen=True)
class BrowserSettings:
    """
    Browser settings data.

    Args:
        name: The name to use for screenshots.
        width: The width of the browser window.
        height: The height of the browser window (use :obj:`None` for unlimited).
        mobile: Whether it is a mobile browser.

    """
    name: Optional[str] = None
    width: int = 1920
    height: Optional[int] = None
    mobile: bool = False


class Browser:
    def __init__(
        self,
        temp_path: Path,
        settings: Optional[BrowserSettings] = None,
        screenshot_path: Path = Path('screenshot'),
    ):
        """
        Browser automation.

        Has the same attributes and methods as a :mod:`selenium.webdriver.Chrome
        <selenium.webdriver.chrome.webdriver>` object.
        """
        self.temp_path = temp_path
        self.settings = settings or BrowserSettings()
        logger.debug(f'Browser settings: {asdict(self.settings)}')
        self.name = self.settings.name
        self.screenshot_path = screenshot_path
        logger.debug(f'Using screenshot path "{self.screenshot_path}"')

        options = cast(Any, ChromeOptions())  # add_argument is not yet typed
        if self.settings.mobile:
            options.add_argument('--hide-scrollbars')
        options.add_argument('--headless')
        options.add_argument(f'--window-size={self.settings.width},{self.settings.height or 1080}')
        self.driver = Browser._get_driver(options=options)

    @staticmethod
    def _get_driver(options: ChromeOptions) -> Chrome:
        try:
            return Chrome(options=options)
        except SessionNotCreatedException as error:
            if driver_version_error := Browser._driver_version_error(str(error.msg)):
                raise RuntimeError(driver_version_error) from None  # override parent exception
            raise

    @staticmethod
    def _driver_version_error(error_message: str) -> Optional[str]:
        search = re.search(
            'This version of ChromeDriver only supports Chrome version (.*?)\n'
            'Current browser version is (.*?) with binary path',
            error_message,
        )
        return '\n'.join([
            f'Invalid ChromeDriver version (compatible with Chrome {search.group(1)})',
            f'Please download a release compatible with Chrome version {search.group(2)} '
            '(see https://chromedriver.chromium.org/downloads)'
        ]) if search else None

    def __enter__(self) -> 'Browser':
        return self

    def __exit__(self, *args: Any) -> None:
        self.driver.__exit__(*args)

    def __getattr__(self, name: str) -> Any:
        return getattr(self.driver, name)

    @contextmanager
    def auto_height(self) -> Iterator[None]:
        if self.settings.height:
            yield
            return
        logger.debug('Using full page height')
        original_size = self.driver.get_window_size()
        script = 'return document.body.parentNode.scrollHeight'
        height = self.driver.execute_script(script)  # type: ignore[no-untyped-call]
        self.driver.set_window_size(original_size['width'], height)
        try:
            yield
        finally:
            self.driver.set_window_size(**original_size)

    def check(self, name: Optional[str] = None) -> None:
        """
        Create a screenshot and check it against an expected version.

        Args:
            name: The name of the expected screenshot.

        """
        name_parts = [self.screenshot_path.name, self.name, name]
        full_name = '_'.join(part for part in name_parts if part is not None)
        expected = self.screenshot_path.with_name(full_name).with_suffix('.png')
        with self.auto_height():
            screenshot = self.driver.get_screenshot_as_png()
        assert_image_equal(actual=screenshot, expected=expected, temp_path=self.temp_path)


@pytest.fixture
def browser(tmp_path: Path, request: Any) -> Iterable[Browser]:
    """
    Yield a :class:`Browser` instance.
    """
    params = getattr(request, 'param', {})
    screenshot_path = params.pop('screenshot_path', None)
    kwargs = {'screenshot_path': screenshot_path} if screenshot_path else {}
    with Browser(temp_path=tmp_path, settings=BrowserSettings(**params), **kwargs) as instance:
        yield instance


def set_browser(*args: Dict[str, Any], **kwargs: Any) -> Fixture[Any]:
    """
    Control the creation of the :class:`Browser` object in the :func:`browser <browser>` fixture.

    Args:
        *args: A list of arguments to forward to the underlying :class:`BrowserSettings` instances.
        **kwargs: The arguments to forward to the underlying :class:`BrowserSettings` instance.

    You can specify browser settings with this decorator as follows:

    .. code-block:: python

        from pytest_django.live_server_helper import LiveServer
        from pytest_logikal.browser import Browser, set_browser


        @set_browser(width=1800, height=900)
        def test_homepage(browser: Browser, live_server: LiveServer) -> None:
            browser.get(live_server.url)
            browser.check()

    You can also run your test in multiple browser environments:

    .. code-block:: python

        from pytest_django.live_server_helper import LiveServer
        from pytest_logikal.browser import Browser, set_browser


        @set_browser(
            dict(name='desktop', width=1800),
            dict(name='mobile', width=600, mobile=True),
        )
        def test_homepage(browser: Browser, live_server: LiveServer) -> None:
            browser.get(live_server.url)
            browser.check()

    """
    argvalues = args or [kwargs]
    for argvalue in argvalues:
        argvalue['name'] = argvalue.get(
            'name', re.sub(r'[{}\[\]\':,]', '', str(argvalue)).replace(' ', '_').lower(),
        )

    def parametrized_test_function(function: Function) -> Any:
        source_file = inspect.getsourcefile(function)
        if not source_file:
            raise RuntimeError(f'Source file cannot be found for "{function}"')
        source_file_path = Path(source_file)

        for argvalue in argvalues:
            argvalue['screenshot_path'] = (
                source_file_path.parent / 'screenshots' / source_file_path.stem / function.__name__
            )

        return pytest.mark.parametrize(
            argnames='browser', argvalues=argvalues, indirect=True, ids=itemgetter('name'),
        )(function)

    return parametrized_test_function



import sys
from io import BytesIO
from logging import getLogger
from operator import methodcaller
from os import getcwd
from pathlib import Path
from shutil import copy
from subprocess import run
from typing import Any, Callable, Optional, Type, TypeVar

from PIL import Image, ImageChops

logger = getLogger(__name__)

Function = TypeVar('Function', bound=Callable[..., Any])


def hide_traceback(function: Function, error: Type[Exception] = AssertionError) -> Function:
    getattr(function, '__globals__')['__tracebackhide__'] = methodcaller('errisinstance', error)
    return function


@hide_traceback
def assert_image_equal(actual: bytes, expected: Path, temp_path: Path) -> None:
    temp_actual_path = temp_path / 'actual.png'
    temp_expected_path = temp_path / 'expected.png'
    temp_diff_path = temp_path / 'diff.png'

    logger.info(f'Checking expected image in "{expected}"')
    if expected.is_file():
        with Image.open(BytesIO(actual)) as actual_image, Image.open(expected) as expected_image:
            if actual_image == expected_image:
                logger.info('The actual image matches the expected image')
                return

            temp_actual_path.write_bytes(actual)  # saving the temporary actual image
            copy(expected, temp_expected_path)  # saving the temporary expected image
            diff_image = ImageChops.invert(ImageChops.difference(
                expected_image.convert('RGB'),
                actual_image.convert('RGB'),
            ))
            diff_image.save(str(temp_diff_path))  # saving the temporary diff image
            save_image_prompt(
                message='Actual image differs from the expected image',
                source=temp_actual_path, destination=expected, difference=temp_diff_path,
            )
    else:
        logger.info('Expected image not found')
        temp_expected_path.write_bytes(actual)  # saving the temporary actual image as expected
        save_image_prompt(
            message='Expected image file does not exist',
            source=temp_expected_path, destination=expected,
        )


def save_image_prompt(
    message: str,
    source: Path,
    destination: Path,
    difference: Optional[Path] = None,
) -> None:
    if not sys.stdin.isatty():
        error_lines = [
            f'{message} and this is not an interactive session (consider using --live)',
            f'  Actual: {source}',
            f'  Expected: {destination}',
        ]
        if difference:
            error_lines.append(f'  Difference: {difference}')
        raise AssertionError('\n'.join(error_lines))
    try:
        short_destination = destination.relative_to(getcwd())
    except ValueError:
        short_destination = destination

    colors = {'red': '\033[31m\033[1m', 'reset': '\033[0m'}
    print(f'\n{colors["red"]}{message}!{colors["reset"]}')
    print(short_destination)

    response = input('> Press "enter" to open or type "s" to skip or "c" to cancel: ')
    if response == '':
        # This subprocess call is secure as it is not using untrusted input
        run(['/usr/bin/xdg-open', str(source)], check=False)  # nosec
    elif response == 's':
        logger.info('Image opening skipped')
    else:
        raise AssertionError('Image opening cancelled')

    response = input('> Type "accept" to accept this version or press "enter" to reject: ')
    if response == 'accept':
        destination.parent.mkdir(parents=True, exist_ok=True)
        copy(source, destination)
        logger.info(f'Image saved at "{short_destination}"')
    else:
        raise AssertionError('Image rejected')



import json
import subprocess
from pathlib import Path

import pytest

from pytest_logikal.file_checker import CachedFileCheckItem, CachedFileCheckPlugin
from pytest_logikal.plugin import ItemRunError


def pytest_addoption(parser: pytest.Parser) -> None:
    group = parser.getgroup('bandit')
    group.addoption('--bandit', action='store_true', default=False, help='run bandit')
    parser.addini('bandit_config_file', help='the Bandit configuration file to use')


def pytest_configure(config: pytest.Config) -> None:
    config.addinivalue_line('markers', 'bandit: tests checking common security issues.')
    if config.option.bandit:
        config.pluginmanager.register(BanditPlugin(config=config))


class BanditItem(CachedFileCheckItem):
    def run(self) -> None:
        severity = {'LOW': 'warning', 'MEDIUM': 'error', 'HIGH': 'critical'}
        config_file = (
            self.config.getini('bandit_config_file')
            or str(Path(__file__).parent / 'bandit_config.yml')
        )
        command = [
            'bandit', '--configfile', config_file, '--quiet', '--format', 'json', str(self.path),
        ]

        # This subprocess call is secure as it is not using untrusted input
        process = subprocess.run(command, capture_output=True, text=True, check=False)  # nosec
        try:
            report = json.loads(process.stdout)
            if report.get('results'):
                raise ItemRunError('\n\n'.join(
                    f'{error["line_number"]}:{error["col_offset"]}: '
                    f'{severity[error["issue_severity"]]}: {error["issue_text"]} '
                    f'({error["test_id"]}: {error["test_name"]}, '
                    f'confidence: {error["issue_confidence"].lower()})\n'
                    f'More info: {error["more_info"]}'
                    for error in report['results']
                ))
        except json.decoder.JSONDecodeError as error:
            raise ItemRunError(f'Error: {process.stdout or process.stderr}') from error


class BanditPlugin(CachedFileCheckPlugin):
    name = 'bandit'
    item = BanditItem



from contextlib import redirect_stdout
from io import StringIO

import pycodestyle
import pydocstyle
import pytest

from pytest_logikal.core import PYPROJECT
from pytest_logikal.file_checker import CachedFileCheckItem, CachedFileCheckPlugin
from pytest_logikal.plugin import ItemRunError


def pytest_addoption(parser: pytest.Parser) -> None:
    group = parser.getgroup('style')
    group.addoption('--style', action='store_true', default=False,
                    help='run pycodestyle and pydocstyle')


def pytest_configure(config: pytest.Config) -> None:
    config.addinivalue_line('markers', 'style: tests checking code and docstring style.')
    if config.option.style:
        config.pluginmanager.register(StylePlugin(config=config))


class StyleItem(CachedFileCheckItem):
    def run(self) -> None:
        # Run pycodestyle
        ignore = ['E133', 'W503']  # mutually exclusive checks with E123 and W504
        max_line_length = int(self.config.getini('max_line_length'))
        with redirect_stdout(StringIO()) as code_stdout:
            code_errors = pycodestyle.Checker(
                filename=self.path,
                max_line_length=max_line_length,
                max_doc_length=max_line_length,
                format='%(row)s:%(col)s: error: %(text)s (%(code)s)',
                ignore=PYPROJECT.get('tool', {}).get('pycodestyle', {}).get('ignore', ignore),
            ).check_all()

        # Run pydocstyle
        select = pydocstyle.violations.all_errors - {
            'D100', 'D101', 'D102', 'D103', 'D104', 'D105', 'D106', 'D107',
            'D200', 'D203', 'D204', 'D212', 'D406', 'D407', 'D408', 'D409',
        }
        select = PYPROJECT.get('tool', {}).get('pydocstyle', {}).get('select', select)
        doc_errors = list(pydocstyle.check(filenames=[str(self.path)], select=select))

        # Report errors
        messages = []
        if code_errors:
            messages.append(code_stdout.getvalue().rstrip())
        if doc_errors:
            messages.extend(f'{e.line}: error: {e.short_desc} ({e.code})' for e in doc_errors)
        if messages:
            raise ItemRunError('\n'.join(messages))


class StylePlugin(CachedFileCheckPlugin):
    name = 'style'
    item = StyleItem



import logging
import os
import shutil
import sys
from importlib.util import find_spec
from pathlib import Path
from typing import Callable, Dict, Iterator, List, Optional, Tuple, Union

import pytest
import tomli

sys.path.insert(0, os.getcwd())

PYPROJECT = (
    tomli.loads(Path('pyproject.toml').read_text(encoding='utf-8'))
    if Path('pyproject.toml').exists() else {}
)
PLUGINS = [
    'mypy', 'bandit', 'build', 'docs', 'isort', 'licenses', 'pylint', 'requirements', 'style',
]

ReportInfoType = Tuple[Union[os.PathLike, str], Optional[int], str]


def pytest_addoption(parser: pytest.Parser) -> None:
    group = parser.getgroup('logikal')
    group.addoption('--live', action='store_true', help='run live tests')
    group.addoption('--fast', action='store_true', help='do not run any additional checks')
    group.addoption('--clear', action='store_true', help='clear cache before running tests')
    group.addoption('--no-defaults', action='store_true', help='do not use our own defaults')
    group.addoption('--no-mypy', action='store_true', help='do not use mypy')
    group.addoption('--no-bandit', action='store_true', help='do not use bandit')
    group.addoption('--no-build', action='store_true', help='do not run build checks')
    group.addoption('--no-docs', action='store_true', help='do not run documentation checks')
    group.addoption('--no-isort', action='store_true', help='do not use isort')
    group.addoption('--no-licenses', action='store_true', help='do not check licenses')
    group.addoption('--no-pylint', action='store_true', help='do not use pylint')
    group.addoption('--no-requirements', action='store_true', help='do not check requirements')
    group.addoption('--no-style', action='store_true', help='do not use pycodestyle & pydocstyle')
    parser.addini('max_line_length', default='99', help='the maximum line length to use')
    parser.addini('cov_fail_under', default='100', help='target coverage percentage')


def pytest_addhooks(pluginmanager: pytest.PytestPluginManager) -> None:
    if not find_spec('selenium'):
        pluginmanager.set_blocked('logikal_browser')
    if not find_spec('pytest_django'):
        pluginmanager.set_blocked('logikal_django')


# Untyped decorator (see https://github.com/pytest-dev/pytest/issues/7469)
@pytest.hookimpl(hookwrapper=True)  # type: ignore[misc]
def pytest_load_initial_conftests(early_config: pytest.Config, args: List[str]) -> Iterator[None]:
    if '--no-defaults' in args:
        yield

    # Updating arguments
    args.extend(['--strict-config', '--strict-markers'])
    namespace = early_config.known_args_namespace
    if '--live' in args:
        args.append('--capture=no')
        namespace.capture = 'no'
    if '--clear' in args:
        args.append('--cache-clear')
        namespace.cacheclear = True
    if '-r' not in args:
        args.extend(['-r', 'fExX'])
    if '-n' not in args:
        args.extend(['-n', 'auto' if '--live' not in args else '0'])
        namespace.dist = 'load' if '--live' not in args else 'no'
    if '--fast' not in args:
        for plugin in PLUGINS:
            if f'--no-{plugin}' not in args:
                args.append(f'--{plugin}')
                setattr(namespace, plugin, True)
    if all(arg not in args for arg in ['--cov', '--no-cov', '--live', '--fast']):
        args.extend(['--cov', '--no-cov-on-fail'])
        namespace.no_cov_on_fail = True

    # Setting coverage options
    if '--cov' in args:
        fail_under = early_config.getini('cov_fail_under')
        namespace.cov_source = ['.']
        namespace.cov_fail_under = float(fail_under) if '.' in fail_under else int(fail_under)
        namespace.cov_report = {'term-missing': 'skip-covered'}

    # Updating config
    ini_defaults: Dict[str, Union[str, List[str]]] = {
        'console_output_style': 'classic',
        'xfail_strict': 'True',
        'filterwarnings': ['error'],
        'log_cli': 'True' if '--live' in args else 'False',
        'log_level': 'DEBUG',
        'log_format': '%(asctime)s.%(msecs)03d %(levelname)s %(message)s (%(name)s:%(lineno)s)',
        'log_date_format': '%Y-%m-%d %H:%M:%S',
        'log_auto_indent': 'True',
    }
    early_config.inicfg = {**ini_defaults, **early_config.inicfg}
    yield


def pytest_configure(config: pytest.Config) -> None:
    # Clearing cache
    if config.getoption('clear'):
        print('Clearing cache')
        Path('.coverage').unlink(missing_ok=True)
        shutil.rmtree('.pytest_cache', ignore_errors=True)
        shutil.rmtree('.mypy_cache', ignore_errors=True)

    # Hiding overly verbose debug and info log messages
    logging.getLogger('faker').setLevel(logging.INFO)
    logging.getLogger('filelock').setLevel(logging.WARNING)
    logging.getLogger('PIL').setLevel(logging.INFO)
    logging.getLogger('pydocstyle').setLevel(logging.WARNING)
    logging.getLogger('matplotlib').setLevel(logging.INFO)

    # Hiding worker information lines
    if not config.getoption('verbose'):
        config.pluginmanager.get_plugin('reports').getworkerinfoline = lambda *_args, **_kwargs: ''

    # Configuring mypy
    mypy = config.pluginmanager.get_plugin('mypy')
    mypy.mypy_argv = [
        '--no-error-summary',
        '--strict',
        '--show-column-numbers',
        '--warn-unreachable',
    ]


def unified_reportinfo(item: pytest.Item, header: str) -> Callable[[], ReportInfoType]:
    def reportinfo() -> ReportInfoType:
        return (item.path, None, header)
    return reportinfo


# Untyped decorator (see https://github.com/pytest-dev/pytest/issues/7469)
@pytest.hookimpl(trylast=True)  # type: ignore[misc]
def pytest_collection_modifyitems(items: List[pytest.Item]) -> None:
    for item in items:
        source = item.nodeid.split('::')[-1].lower()
        if source == 'mypy':
            header = f'[{item.name}] {item.path.relative_to(item.config.invocation_params.dir)}'
            setattr(item, 'reportinfo', unified_reportinfo(item=item, header=header))
        if source == 'mypy-status':
            setattr(item, '_nodeid', '::check::mypy-status')
            setattr(item, 'reportinfo', unified_reportinfo(item=item, header='[mypy-status]'))



import shutil
import subprocess
from pathlib import Path

import pytest

from pytest_logikal.core import PYPROJECT
from pytest_logikal.file_checker import CachedFileCheckItem, CachedFileCheckPlugin
from pytest_logikal.plugin import ItemRunError


def pytest_addoption(parser: pytest.Parser) -> None:
    group = parser.getgroup('build')
    group.addoption('--build', action='store_true', default=False, help='run build checks')


def pytest_configure(config: pytest.Config) -> None:
    config.addinivalue_line('markers', 'build: tests package building.')
    if config.option.build:
        config.pluginmanager.register(BuildPlugin(config=config))


class BuildItem(CachedFileCheckItem):
    def run(self) -> None:
        # Deleting existing distributions
        shutil.rmtree('dist', ignore_errors=True)

        # Checking build process
        process = subprocess.run(  # nosec: secure, not using untrusted input
            ['python3', '-m', 'build', '--sdist', '--wheel'],
            capture_output=True, text=True, check=False,
        )
        errors = (process.stderr or process.stdout) if process.returncode != 0 else process.stderr
        cleaned_errors = [
            error for error in (errors.strip().split('\n') if errors else [])
            if '_BetaConfiguration' not in error and 'warnings.warn(' not in error
            and 'shallow and may cause errors' not in error
        ]
        if cleaned_errors:
            raise ItemRunError('\n'.join(cleaned_errors))

        # Checking distribution
        process = subprocess.run(  # nosec: secure, not using untrusted input
            ['twine', 'check', '--strict', 'dist/*'], capture_output=True, text=True, check=False,
        )
        if process.returncode != 0:
            raise ItemRunError(process.stdout or process.stderr)


class BuildPlugin(CachedFileCheckPlugin):
    name = 'build'
    item = BuildItem

    def check_file(self, file_path: Path) -> bool:
        if file_path.relative_to(self.config.invocation_params.dir) == Path('pyproject.toml'):
            return 'build-system' in PYPROJECT
        return False



from typing import Any

import pytest
from mypy_django_plugin import config as mypy_django_plugin_config


def pytest_configure(config: pytest.Config) -> None:
    # Patching django-stubs
    def parse_toml_file(self: Any, *_args: Any, **_kwargs: Any) -> None:
        self.django_settings_module = str(config.inicfg['DJANGO_SETTINGS_MODULE'])

    mypy_django_plugin_config.DjangoPluginConfig.parse_toml_file = parse_toml_file  # type: ignore
